@@ -5,10 +5,16 @@
 
 #include "xf86Cursor.h"
 
+#ifdef XvExtension
 #include "xf86xv.h"
 #include <X11/extensions/Xv.h>
+#endif
 #include <string.h>
 
+#include "compat-api.h"
+
+#define DUMMY_MAX_SCREENS 16
+
 /* Supported chipsets */
 typedef enum {
     DUMMY_CHIP
@@ -16,20 +22,14 @@ typedef enum {
 
 /* function prototypes */
 
-extern Bool DUMMYSwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
-extern void DUMMYAdjustFrame(int scrnIndex, int x, int y, int flags);
+extern Bool DUMMYSwitchMode(SWITCH_MODE_ARGS_DECL);
+extern void DUMMYAdjustFrame(ADJUST_FRAME_ARGS_DECL);
 
 /* in dummy_cursor.c */
 extern Bool DUMMYCursorInit(ScreenPtr pScrn);
 extern void DUMMYShowCursor(ScrnInfoPtr pScrn);
 extern void DUMMYHideCursor(ScrnInfoPtr pScrn);
 
-/* in dummy_dga.c */
-Bool DUMMYDGAInit(ScreenPtr pScreen);
-
-/* in dummy_video.c */
-extern void DUMMYInitVideo(ScreenPtr pScreen);
-
 /* globals */
 typedef struct _color
 {
@@ -40,10 +40,6 @@ typedef struct _color
 
 typedef struct dummyRec 
 {
-    DGAModePtr		DGAModes;
-    int			numDGAModes;
-    Bool		DGAactive;
-    int			DGAViewportStatus;
     /* options */
     OptionInfoPtr Options;
     Bool swCursor;
@@ -55,17 +51,15 @@ typedef struct dummyRec
     int cursorX, cursorY;
     int cursorFG, cursorBG;
 
-    Bool screenSaver;
-    Bool video;
-    XF86VideoAdaptorPtr overlayAdaptor;
-    int overlay;
-    int overlay_offset;
-    int videoKey;
-    int interlace;
-    dummy_colors colors[256];
-    pointer* FBBase;
-    Bool        (*CreateWindow)() ;     /* wrapped CreateWindow */
+    dummy_colors colors[1024];
+    Bool        (*CreateWindow)(WindowPtr) ;     /* wrapped CreateWindow */
     Bool prop;
+    /* XRANDR support begin */
+    int num_screens;
+    struct _xf86Crtc *paCrtcs[DUMMY_MAX_SCREENS];
+    struct _xf86Output *paOutputs[DUMMY_MAX_SCREENS];
+    int connected_outputs;
+    /* XRANDR support end */
 } DUMMYRec, *DUMMYPtr;
 
 /* The privates of the DUMMY driver */
