@@ -14,9 +14,6 @@
 /* All drivers initialising the SW cursor need this */
 #include "mipointer.h"
 
-/* All drivers implementing backing store need this */
-#include "mibstore.h"
-
 /* All drivers using the mi colormap manipulation need this */
 #include "micmap.h"
 
@@ -32,8 +29,7 @@
 
 #include "picturestr.h"
 
-#include "xf86xv.h"
-#include <X11/extensions/Xv.h>
+#include "xf86Crtc.h"
 
 /*
  * Driver data structures.
@@ -45,31 +41,23 @@
 #include <X11/Xproto.h>
 #include "scrnintstr.h"
 #include "servermd.h"
-#ifdef XFreeXDGA
-#define _XF86DGA_SERVER_
-#include <X11/extensions/xf86dgastr.h>
-#endif
 
 /* Mandatory functions */
 static const OptionInfoRec *	DUMMYAvailableOptions(int chipid, int busid);
 static void     DUMMYIdentify(int flags);
 static Bool     DUMMYProbe(DriverPtr drv, int flags);
 static Bool     DUMMYPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool     DUMMYScreenInit(int Index, ScreenPtr pScreen, int argc,
-                                  char **argv);
-static Bool     DUMMYEnterVT(int scrnIndex, int flags);
-static void     DUMMYLeaveVT(int scrnIndex, int flags);
-static Bool     DUMMYCloseScreen(int scrnIndex, ScreenPtr pScreen);
+static Bool     DUMMYScreenInit(SCREEN_INIT_ARGS_DECL);
+static Bool     DUMMYEnterVT(VT_FUNC_ARGS_DECL);
+static void     DUMMYLeaveVT(VT_FUNC_ARGS_DECL);
+static Bool     DUMMYCloseScreen(CLOSE_SCREEN_ARGS_DECL);
 static Bool     DUMMYCreateWindow(WindowPtr pWin);
-static void     DUMMYFreeScreen(int scrnIndex, int flags);
-static ModeStatus DUMMYValidMode(int scrnIndex, DisplayModePtr mode,
+static void     DUMMYFreeScreen(FREE_SCREEN_ARGS_DECL);
+static ModeStatus DUMMYValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode,
                                  Bool verbose, int flags);
 static Bool	DUMMYSaveScreen(ScreenPtr pScreen, int mode);
 
 /* Internally used functions */
-static Bool     dummyModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode);
-static void	dummySave(ScrnInfoPtr pScrn);
-static void	dummyRestore(ScrnInfoPtr pScrn, Bool restoreText);
 static Bool	dummyDriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op,
 				pointer ptr);
 
@@ -85,12 +73,19 @@ static Bool	dummyDriverFunc(ScrnInfoPtr pScrn, xorgDriverFuncOp op,
 #define DUMMY_MINOR_VERSION PACKAGE_VERSION_MINOR
 #define DUMMY_PATCHLEVEL PACKAGE_VERSION_PATCHLEVEL
 
+#define DUMMY_MAX_WIDTH 32767
+#define DUMMY_MAX_HEIGHT 32767
+
 /*
  * This is intentionally screen-independent.  It indicates the binding
  * choice made in the first PreInit.
  */
 static int pix24bpp = 0;
 
+Atom width_mm_atom = 0;
+#define WIDTH_MM_NAME  "WIDTH_MM"
+Atom height_mm_atom = 0;
+#define HEIGHT_MM_NAME "HEIGHT_MM"
 
 /*
  * This contains the functions needed by the server after loading the driver
@@ -143,6 +138,271 @@ static XF86ModuleVersionInfo dummyVersRec =
 	{0,0,0,0}
 };
 
+
+/************************
+ * XRANDR support begin *
+ ************************/
+
+static Bool dummy_config_resize(ScrnInfoPtr pScrn, int cw, int ch);
+static Bool DUMMYAdjustScreenPixmap(ScrnInfoPtr pScrn, int width, int height);
+
+static const xf86CrtcConfigFuncsRec DUMMYCrtcConfigFuncs = {
+    .resize = dummy_config_resize
+};
+
+
+static void
+dummy_crtc_dpms(xf86CrtcPtr crtc, int mode)
+{
+}
+
+static Bool
+dummy_crtc_lock (xf86CrtcPtr crtc)
+{
+    return FALSE;
+}
+
+static Bool
+dummy_crtc_mode_fixup (xf86CrtcPtr crtc, DisplayModePtr mode,
+                              DisplayModePtr adjusted_mode)
+{
+    return TRUE;
+}
+
+static void
+dummy_crtc_stub (xf86CrtcPtr crtc)
+{
+}
+
+static void
+dummy_crtc_gamma_set (xf86CrtcPtr crtc, CARD16 *red,
+                             CARD16 *green, CARD16 *blue, int size)
+{
+}
+
+static void *
+dummy_crtc_shadow_allocate (xf86CrtcPtr crtc, int width, int height)
+{
+    return NULL;
+}
+
+static void
+dummy_crtc_mode_set (xf86CrtcPtr crtc, DisplayModePtr mode,
+                            DisplayModePtr adjusted_mode, int x, int y)
+{
+}
+
+static const xf86CrtcFuncsRec DUMMYCrtcFuncs = {
+    .dpms = dummy_crtc_dpms,
+    .save = NULL, /* These two are never called by the server. */
+    .restore = NULL,
+    .lock = dummy_crtc_lock,
+    .unlock = NULL, /* This will not be invoked if lock returns FALSE. */
+    .mode_fixup = dummy_crtc_mode_fixup,
+    .prepare = dummy_crtc_stub,
+    .mode_set = dummy_crtc_mode_set,
+    .commit = dummy_crtc_stub,
+    .gamma_set = dummy_crtc_gamma_set,
+    .shadow_allocate = dummy_crtc_shadow_allocate,
+    .shadow_create = NULL, /* These two should not be invoked if allocate
+                              returns NULL. */
+    .shadow_destroy = NULL,
+    .set_cursor_colors = NULL,
+    .set_cursor_position = NULL,
+    .show_cursor = NULL,
+    .hide_cursor = NULL,
+    .load_cursor_argb = NULL,
+    .destroy = dummy_crtc_stub
+};
+
+static void
+dummy_output_stub (xf86OutputPtr output)
+{
+}
+
+static void
+dummy_output_dpms (xf86OutputPtr output, int mode)
+{
+}
+
+static int
+dummy_output_mode_valid (xf86OutputPtr output, DisplayModePtr mode)
+{
+    return MODE_OK;
+}
+
+static Bool
+dummy_output_mode_fixup (xf86OutputPtr output, DisplayModePtr mode,
+        DisplayModePtr adjusted_mode)
+{
+    return TRUE;
+}
+
+static void
+dummy_output_mode_set (xf86OutputPtr output, DisplayModePtr mode,
+        DisplayModePtr adjusted_mode)
+{
+    DUMMYPtr dPtr = DUMMYPTR(output->scrn);
+    int index = (int64_t)output->driver_private;
+
+    /* set to connected at first mode set */
+    dPtr->connected_outputs |= 1 << index;
+}
+
+/* The first virtual monitor is always connected. Others only after setting its
+ * mode */
+static xf86OutputStatus
+dummy_output_detect (xf86OutputPtr output)
+{
+    DUMMYPtr dPtr = DUMMYPTR(output->scrn);
+    int index = (int64_t)output->driver_private;
+
+    if (dPtr->connected_outputs & (1 << index))
+        return XF86OutputStatusConnected;
+    else
+        return XF86OutputStatusDisconnected;
+}
+
+static DisplayModePtr
+dummy_output_get_modes (xf86OutputPtr output)
+{
+    DisplayModePtr pModes = NULL, pMode, pModeSrc;
+
+    /* copy modes from config */
+    for (pModeSrc = output->scrn->modes; pModeSrc; pModeSrc = pModeSrc->next)
+    {
+            pMode = xnfcalloc(1, sizeof(DisplayModeRec));
+            memcpy(pMode, pModeSrc, sizeof(DisplayModeRec));
+            pMode->next = NULL;
+            pMode->prev = NULL;
+            pMode->name = strdup(pModeSrc->name);
+            pModes = xf86ModesAdd(pModes, pMode);
+            if (pModeSrc->next == output->scrn->modes)
+                break;
+    }
+    return pModes;
+}
+
+void dummy_output_register_prop(xf86OutputPtr output, Atom prop, uint64_t value)
+{
+    INT32 dims_range[2] = { 0, 65535 };
+    int err;
+
+    err = RRConfigureOutputProperty(output->randr_output, prop, FALSE,
+            TRUE, FALSE, 2, dims_range);
+    if (err != 0)
+        xf86DrvMsg(output->scrn->scrnIndex, X_ERROR,
+                                       "RRConfigureOutputProperty error, %d\n", err);
+
+    err = RRChangeOutputProperty(output->randr_output, prop, XA_INTEGER,
+            32, PropModeReplace, 1, &value, FALSE, FALSE);
+    if (err != 0)
+        xf86DrvMsg(output->scrn->scrnIndex, X_ERROR,
+                "RRChangeOutputProperty error, %d\n", err);
+}
+
+void dummy_output_create_resources(xf86OutputPtr output)
+{
+    if (!ValidAtom(width_mm_atom))
+        width_mm_atom = MakeAtom(WIDTH_MM_NAME, strlen(WIDTH_MM_NAME), 1);
+    if (!ValidAtom(height_mm_atom))
+        height_mm_atom = MakeAtom(HEIGHT_MM_NAME, strlen(HEIGHT_MM_NAME), 1);
+
+    dummy_output_register_prop(output, width_mm_atom, 0);
+    dummy_output_register_prop(output, height_mm_atom, 0);
+}
+
+static Bool dummy_output_set_property(xf86OutputPtr output, Atom property,
+        RRPropertyValuePtr value)
+{
+
+    if (property == width_mm_atom || property == height_mm_atom) {
+        INT32 val;
+
+        if (value->type != XA_INTEGER || value->format != 32 ||
+                value->size != 1)
+        {
+            return FALSE;
+        }
+
+        val = *(INT32 *)value->data;
+        if (property == width_mm_atom)
+            output->mm_width = val;
+        else if (property == height_mm_atom)
+            output->mm_height = val;
+        return TRUE;
+    }
+    return TRUE;
+}
+
+
+static const xf86OutputFuncsRec DUMMYOutputFuncs = {
+    .create_resources = dummy_output_create_resources,
+    .dpms = dummy_output_dpms,
+    .save = NULL, /* These two are never called by the server. */
+    .restore = NULL,
+    .mode_valid = dummy_output_mode_valid,
+    .mode_fixup = dummy_output_mode_fixup,
+    .prepare = dummy_output_stub,
+    .commit = dummy_output_stub,
+    .mode_set = dummy_output_mode_set,
+    .detect = dummy_output_detect,
+    .get_modes = dummy_output_get_modes,
+#ifdef RANDR_12_INTERFACE
+    .set_property = dummy_output_set_property,
+#endif
+    .destroy = dummy_output_stub
+};
+
+static Bool
+dummy_config_resize(ScrnInfoPtr pScrn, int cw, int ch)
+{
+    if (!pScrn->vtSema) {
+        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+                "We do not own the active VT, exiting.\n");
+        return TRUE;
+    }
+    return DUMMYAdjustScreenPixmap(pScrn, cw, ch);
+}
+
+Bool DUMMYAdjustScreenPixmap(ScrnInfoPtr pScrn, int width, int height)
+{
+    ScreenPtr pScreen = pScrn->pScreen;
+    PixmapPtr pPixmap = pScreen->GetScreenPixmap(pScreen);
+    DUMMYPtr dPtr = DUMMYPTR(pScrn);
+    uint64_t cbLine = (width * xf86GetBppFromDepth(pScrn, pScrn->depth) / 8 + 3) & ~3;
+    int displayWidth = cbLine * 8 / xf86GetBppFromDepth(pScrn, pScrn->depth);
+
+    if (   width == pScrn->virtualX
+            && height == pScrn->virtualY
+            && displayWidth == pScrn->displayWidth)
+        return TRUE;
+    if (!pPixmap) {
+        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+                "Failed to get the screen pixmap.\n");
+        return FALSE;
+    }
+    if (cbLine > UINT32_MAX || cbLine * height >= pScrn->videoRam * 1024)
+    {
+        xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+                "Unable to set up a virtual screen size of %dx%d with %d Kb of video memory available.  Please increase the video memory size.\n",
+                width, height, pScrn->videoRam);
+        return FALSE;
+    }
+    pScreen->ModifyPixmapHeader(pPixmap, width, height,
+            pScrn->depth, xf86GetBppFromDepth(pScrn, pScrn->depth), cbLine,
+            pPixmap->devPrivate.ptr);
+    pScrn->virtualX = width;
+    pScrn->virtualY = height;
+    pScrn->displayWidth = displayWidth;
+
+    return TRUE;
+}
+
+/**********************
+ * XRANDR support end *
+ **********************/
+
 /*
  * This is the module init data.
  * Its name has to be the driver name followed by ModuleData
@@ -191,7 +451,7 @@ DUMMYGetRec(ScrnInfoPtr pScrn)
 
     if (pScrn->driverPrivate == NULL)
 	return FALSE;
-        return TRUE;
+    return TRUE;
 }
 
 static void
@@ -199,7 +459,7 @@ DUMMYFreeRec(ScrnInfoPtr pScrn)
 {
     if (pScrn->driverPrivate == NULL)
 	return;
-    xfree(pScrn->driverPrivate);
+    free(pScrn->driverPrivate);
     pScrn->driverPrivate = NULL;
 }
 
@@ -265,6 +525,9 @@ DUMMYProbe(DriverPtr drv, int flags)
 	    }
 	}
     }    
+
+    free(devSections);
+
     return foundScreen;
 }
 
@@ -280,7 +543,7 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
     ClockRangePtr clockRanges;
     int i;
     DUMMYPtr dPtr;
-    int maxClock = 230000;
+    int maxClock = 300000;
     GDevPtr device = xf86GetEntityInfo(pScrn->entityList[0])->device;
 
     if (flags & PROBE_DETECT) 
@@ -309,6 +572,7 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
 	case 15:
 	case 16:
 	case 24:
+	case 30:
 	    break;
 	default:
 	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
@@ -354,7 +618,7 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
 
     xf86CollectOptions(pScrn, device->options);
     /* Process the options */
-    if (!(dPtr->Options = xalloc(sizeof(DUMMYOptions))))
+    if (!(dPtr->Options = malloc(sizeof(DUMMYOptions))))
 	return FALSE;
     memcpy(dPtr->Options, DUMMYOptions, sizeof(DUMMYOptions));
 
@@ -390,7 +654,7 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
     clockRanges->next = NULL;
     clockRanges->ClockMulFactor = 1;
     clockRanges->minClock = 11000;   /* guessed §§§ */
-    clockRanges->maxClock = 300000;
+    clockRanges->maxClock = maxClock;
     clockRanges->clockIndex = -1;		/* programmable */
     clockRanges->interlaceAllowed = TRUE; 
     clockRanges->doubleScanAllowed = TRUE;
@@ -402,8 +666,9 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
 	int apertureSize = (pScrn->videoRam * 1024);
 	i = xf86ValidateModes(pScrn, pScrn->monitor->Modes,
 			      pScrn->display->modes, clockRanges,
-			      NULL, 256, 2048,(8 * pScrn->bitsPerPixel),
-			      128, 2048, pScrn->display->virtualX,
+			      NULL, 256, DUMMY_MAX_WIDTH,
+			      (8 * pScrn->bitsPerPixel),
+			      128, DUMMY_MAX_HEIGHT, pScrn->display->virtualX,
 			      pScrn->display->virtualY, apertureSize,
 			      LOOKUP_BEST_REFRESH);
 
@@ -457,25 +722,15 @@ DUMMYPreInit(ScrnInfoPtr pScrn, int flags)
 
 /* Mandatory */
 static Bool
-DUMMYEnterVT(int scrnIndex, int flags)
+DUMMYEnterVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
-    
-    /* Should we re-save the text mode on each VT enter? */
-    if(!dummyModeInit(pScrn, pScrn->currentMode))
-      return FALSE;
-
-    DUMMYAdjustFrame(pScrn->scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);    
-
     return TRUE;
 }
 
 /* Mandatory */
 static void
-DUMMYLeaveVT(int scrnIndex, int flags)
+DUMMYLeaveVT(VT_FUNC_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
-    dummyRestore(pScrn, TRUE);
 }
 
 static void
@@ -515,33 +770,25 @@ static ScrnInfoPtr DUMMYScrn; /* static-globalize it */
 
 /* Mandatory */
 static Bool
-DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+DUMMYScreenInit(SCREEN_INIT_ARGS_DECL)
 {
     ScrnInfoPtr pScrn;
     DUMMYPtr dPtr;
     int ret;
     VisualPtr visual;
-    
+    void *pixels;
+
     /*
      * we need to get the ScrnInfoRec for this screen, so let's allocate
      * one first thing
      */
-    pScrn = xf86Screens[pScreen->myNum];
+    pScrn = xf86ScreenToScrn(pScreen);
     dPtr = DUMMYPTR(pScrn);
     DUMMYScrn = pScrn;
 
 
-    if (!(dPtr->FBBase = xalloc(pScrn->videoRam * 1024)))
+    if (!(pixels = malloc(pScrn->videoRam * 1024)))
 	return FALSE;
-    
-    /*
-     * next we save the current state and setup the first mode
-     */
-    dummySave(pScrn);
-    
-    if (!dummyModeInit(pScrn,pScrn->currentMode))
-	return FALSE;
-    DUMMYAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
 
     /*
      * Reset visual list.
@@ -551,17 +798,22 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     /* Setup the visuals we support. */
     
     if (!miSetVisualTypes(pScrn->depth,
-      		      miGetDefaultVisualMask(pScrn->depth),
-		      pScrn->rgbBits, pScrn->defaultVisual))
-         return FALSE;
+                          miGetDefaultVisualMask(pScrn->depth),
+                          pScrn->rgbBits, pScrn->defaultVisual)) {
+        free(pixels);
+        return FALSE;
+    }
 
-    if (!miSetPixmapDepths ()) return FALSE;
+    if (!miSetPixmapDepths ()) {
+        free(pixels);
+        return FALSE;
+    }
 
     /*
      * Call the framebuffer layer's ScreenInit function, and fill in other
      * pScreen fields.
      */
-    ret = fbScreenInit(pScreen, dPtr->FBBase,
+    ret = fbScreenInit(pScreen, pixels,
 			    pScrn->virtualX, pScrn->virtualY,
 			    pScrn->xDpi, pScrn->yDpi,
 			    pScrn->displayWidth, pScrn->bitsPerPixel);
@@ -588,10 +840,58 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
     xf86SetBlackWhitePixels(pScreen);
 
-    DUMMYDGAInit(pScreen);
-    
+    /* initialize XRANDR */
+    xf86CrtcConfigInit(pScrn, &DUMMYCrtcConfigFuncs);
+    /* FIXME */
+    dPtr->num_screens = DUMMY_MAX_SCREENS;
+
+    for (int i=0; i < dPtr->num_screens; i++) {
+        char szOutput[256];
+
+        dPtr->paCrtcs[i] = xf86CrtcCreate(pScrn, &DUMMYCrtcFuncs);
+        dPtr->paCrtcs[i]->driver_private = (void *)(uintptr_t)i;
+
+        /* Set up our virtual outputs. */
+        snprintf(szOutput, sizeof(szOutput), "DUMMY%u", i);
+        dPtr->paOutputs[i] = xf86OutputCreate(pScrn, &DUMMYOutputFuncs,
+                szOutput);
+
+
+        xf86OutputUseScreenMonitor(dPtr->paOutputs[i], FALSE);
+        dPtr->paOutputs[i]->possible_crtcs = 1 << i;
+        dPtr->paOutputs[i]->possible_clones = 0;
+        dPtr->paOutputs[i]->driver_private = (void *)(uintptr_t)i;
+        xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Created crtc (%p) and output %s (%p)\n",
+                (void *)dPtr->paCrtcs[i], szOutput,
+                (void *)dPtr->paOutputs[i]);
+
+    }
+
+    /* bitmask */
+    dPtr->connected_outputs = 1;
+
+    xf86CrtcSetSizeRange(pScrn, 64, 64, DUMMY_MAX_WIDTH, DUMMY_MAX_HEIGHT);
+
+
+    /* Now create our initial CRTC/output configuration. */
+    if (!xf86InitialConfiguration(pScrn, TRUE)) {
+        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Initial CRTC configuration failed!\n");
+        return (FALSE);
+    }
+
+    /* Initialise randr 1.2 mode-setting functions and set first mode.
+     * Note that the mode won't be usable until the server has resized the
+     * framebuffer to something reasonable. */
+    if (!xf86CrtcScreenInit(pScreen)) {
+        return FALSE;
+    }
+    if (!xf86SetDesiredModes(pScrn)) {
+        return FALSE;
+    }
+
+    /* XRANDR initialization end */
     if (dPtr->swCursor)
-	xf86DrvMsg(scrnIndex, X_CONFIG, "Using Software Cursor.\n");
+	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Using Software Cursor.\n");
 
     {
 
@@ -610,7 +910,6 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 		   , lines - pScrn->virtualY);
     }
 
-    miInitializeBackingStore(pScreen);
     xf86SetBackingStore(pScreen);
     xf86SetSilkenMouse(pScreen);
 	
@@ -621,7 +920,7 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     if (!dPtr->swCursor) {
       /* HW cursor functions */
       if (!DUMMYCursorInit(pScreen)) {
-	  xf86DrvMsg(scrnIndex, X_ERROR,
+	  xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 		     "Hardware cursor initialization failed\n");
 	  return FALSE;
       }
@@ -631,14 +930,12 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     if(!miCreateDefColormap(pScreen))
 	return FALSE;
 
-    if (!xf86HandleColormaps(pScreen, 256, pScrn->rgbBits,
+    if (!xf86HandleColormaps(pScreen, 1024, pScrn->rgbBits,
                          DUMMYLoadPalette, NULL, 
                          CMAP_PALETTED_TRUECOLOR 
 			     | CMAP_RELOAD_ON_MODE_SWITCH))
 	return FALSE;
 
-/*     DUMMYInitVideo(pScreen); */
-
     pScreen->SaveScreen = DUMMYSaveScreen;
 
     
@@ -660,105 +957,55 @@ DUMMYScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
 /* Mandatory */
 Bool
-DUMMYSwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
+DUMMYSwitchMode(SWITCH_MODE_ARGS_DECL)
 {
-    return dummyModeInit(xf86Screens[scrnIndex], mode);
+    return TRUE;
 }
 
 /* Mandatory */
 void
-DUMMYAdjustFrame(int scrnIndex, int x, int y, int flags)
+DUMMYAdjustFrame(ADJUST_FRAME_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn;
-    int Base; 
-
-    pScrn = xf86Screens[scrnIndex];
-
-    Base = (y * pScrn->displayWidth + x) >> 2;
-
-    /* Scale Base by the number of bytes per pixel. */
-    switch (pScrn->depth) {
-    case  8 :
-	break;
-    case 15 :
-    case 16 :
-	Base *= 2;
-	break;
-    case 24 :
-	Base *= 3;
-	break;
-    default :
-	break;
-    }
 }
 
 /* Mandatory */
 static Bool
-DUMMYCloseScreen(int scrnIndex, ScreenPtr pScreen)
+DUMMYCloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-    ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
     DUMMYPtr dPtr = DUMMYPTR(pScrn);
 
-    if(pScrn->vtSema){
- 	dummyRestore(pScrn, TRUE);
-	xfree(dPtr->FBBase);
-    }
+    free(pScreen->GetScreenPixmap(pScreen)->devPrivate.ptr);
 
     if (dPtr->CursorInfo)
 	xf86DestroyCursorInfoRec(dPtr->CursorInfo);
 
     pScrn->vtSema = FALSE;
     pScreen->CloseScreen = dPtr->CloseScreen;
-    return (*pScreen->CloseScreen)(scrnIndex, pScreen);
+    return (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 }
 
 /* Optional */
 static void
-DUMMYFreeScreen(int scrnIndex, int flags)
+DUMMYFreeScreen(FREE_SCREEN_ARGS_DECL)
 {
-    DUMMYFreeRec(xf86Screens[scrnIndex]);
+    SCRN_INFO_PTR(arg);
+    DUMMYFreeRec(pScrn);
 }
 
 static Bool
 DUMMYSaveScreen(ScreenPtr pScreen, int mode)
 {
-    ScrnInfoPtr pScrn = NULL;
-    DUMMYPtr dPtr;
-
-    if (pScreen != NULL) {
-	pScrn = xf86Screens[pScreen->myNum];
-	dPtr = DUMMYPTR(pScrn);
-
-	dPtr->screenSaver = xf86IsUnblank(mode);
-    } 
     return TRUE;
 }
 
 /* Optional */
 static ModeStatus
-DUMMYValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
+DUMMYValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose, int flags)
 {
     return(MODE_OK);
 }
 
-static void
-dummySave(ScrnInfoPtr pScrn)
-{
-}
-
-static void 
-dummyRestore(ScrnInfoPtr pScrn, Bool restoreText)
-{
-}
-    
-static Bool
-dummyModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
-{
-    dummyRestore(pScrn, FALSE);
-    
-    return(TRUE);
-}
-
 Atom VFB_PROP  = 0;
 #define  VFB_PROP_NAME  "VFB_IDENT"
 
@@ -787,8 +1034,9 @@ DUMMYCreateWindow(WindowPtr pWin)
         if (! ValidAtom(VFB_PROP))
             VFB_PROP = MakeAtom(VFB_PROP_NAME, strlen(VFB_PROP_NAME), 1);
 
-        ret = ChangeWindowProperty(pWinRoot, VFB_PROP, XA_STRING, 
-		8, PropModeReplace, (int)4, (pointer)"TRUE", FALSE);
+        ret = dixChangeWindowProperty(serverClient, pWinRoot, VFB_PROP,
+                                      XA_STRING, 8, PropModeReplace,
+                                      (int)4, (pointer)"TRUE", FALSE);
 	if( ret != Success)
 		ErrorF("Could not set VFB root window property");
         dPtr->prop = TRUE;
